#include "libscl.h"

using namespace std;
using namespace scl;

int main(int argc, char** argp, char** envp)
{
  INTEGER n = 3;
  realmat A(n,n);

  A(1,1) =  2.66472709297580379e-01;
  A(2,1) = -7.74057796396597220e-02;
  A(3,1) = -3.12433639763852522e+00;
  A(1,2) = -1.60479356329855036e-02;
  A(2,2) =  6.04624824655081994e-02;
  A(3,2) = -8.31530780153308668e-02;
  A(1,3) =  4.98687839396794383e-02;
  A(2,3) =  3.06025448099834341e-01;
  A(3,3) = -1.29392168343915481e-01;

  cout << starbox("/Input matrix A//");
  cout << A;

  realmat E;
  INTEGER ier;
  REAL max_val;

  max_val = eigen(A,E,ier);
  
  cout << starbox("/Results from call to first eigen function//");
  cout << '\n';
  cout << "\tMaximum eigen value = " << fmt('f',20,8,max_val) << '\n';
  cout << '\n';
  cout << "\tError code = " << ier << '\n';
  cout << '\n';
  cout << "\tEigen values = " << '\n';
  for (INTEGER i=1; i<=n; ++i) {
    cout << '\t' << COMPLEX( E[2*(i-1)+1], E[2*(i-1)+2] ) << '\n';
  }

  realmat X;
  max_val = eigen(A,E,X,ier);
  
  cout << starbox("/Results from call to second eigen function//");
  cout << '\n';
  cout << "\tMaximum eigen value = " << fmt('f',20,8,max_val) << '\n';
  cout << '\n';
  cout << "\tError code = " << ier << '\n';
  cout << '\n';
  cout << "\tEigen values = " << '\n';
  for (INTEGER i=1; i<=n; ++i) {
    cout << '\t' << COMPLEX( E[2*(i-1)+1], E[2*(i-1)+2] ) << '\n';
  }
  cout << '\n';
  cout << "\tEigen vectors = " << '\n';
  for (INTEGER j=1; j<=n; ++j) {
    cout << "\tCol " << j << '\n';
    for (INTEGER i=1; i<=n; ++i) {
      cout << '\t' << COMPLEX( X(2*(i-1)+1,j), X(2*(i-1)+2,j) ) << '\n';
    }
  }
  cout << '\n';
  cout << "\tEigen vectors - A * Eigen vectors = " << '\n';
  for (INTEGER j=1; j<=n; ++j) {
    cout << "\tCol " << j << '\n';
    for (INTEGER i=1; i<=n; ++i) {
      COMPLEX sum(0.0,0.0);
      for (INTEGER k=1; k<=n; ++k) {
        sum += COMPLEX(A(i,k),0.0)*COMPLEX(X(2*(k-1)+1,j),X(2*(k-1)+2,j));
      }
      COMPLEX X0 = COMPLEX(X(2*(i-1)+1,j), X(2*(i-1)+2,j));
      COMPLEX X1 = sum/COMPLEX(E[2*(j-1)+1], E[2*(j-1)+2]);
      //cout << '\t' << X0 << "   " << X1 << "   " << X0 - X1 << '\n';
      cout << '\t' << X0 - X1 << '\n';
    }
  }

  n = 14;
  A.resize(n,n);

  /*
  INT_32BIT seed = 9875;
  for (INTEGER i=1; i<=n; ++i) {
    for (INTEGER j=1; j<=n; ++j) {
      cout << "A(" << i << ',' << j << ") = " <<fmt('f',10,5,unsk(seed))<<'\n';
    }
  }
  */
  
  A(1,1) =   -0.93939;
  A(1,2) =    0.87683;
  A(1,3) =   -0.22005;
  A(1,4) =   -1.37321;
  A(1,5) =    0.96858;
  A(1,6) =   -0.14962;
  A(1,7) =   -0.56231;
  A(1,8) =    0.84358;
  A(1,9) =    0.50299;
  A(1,10) =   -0.39677;
  A(1,11) =    0.30035;
  A(1,12) =    0.76154;
  A(1,13) =    0.62041;
  A(1,14) =   -0.70230;
  A(2,1) =    0.62808;
  A(2,2) =   -0.07470;
  A(2,3) =    1.27694;
  A(2,4) =   -0.55836;
  A(2,5) =   -0.68863;
  A(2,6) =   -1.70460;
  A(2,7) =   -1.28454;
  A(2,8) =   -0.22256;
  A(2,9) =   -2.27332;
  A(2,10) =   -0.34231;
  A(2,11) =   -0.65448;
  A(2,12) =    0.76878;
  A(2,13) =    1.19100;
  A(2,14) =   -0.00965;
  A(3,1) =    0.44009;
  A(3,2) =   -0.44663;
  A(3,3) =    0.39962;
  A(3,4) =    0.91864;
  A(3,5) =    0.48439;
  A(3,6) =   -1.61459;
  A(3,7) =    0.36005;
  A(3,8) =   -0.21355;
  A(3,9) =   -0.02187;
  A(3,10) =   -1.00708;
  A(3,11) =   -1.55536;
  A(3,12) =   -1.59166;
  A(3,13) =    1.02600;
  A(3,14) =    0.18776;
  A(4,1) =   -0.97791;
  A(4,2) =   -0.13406;
  A(4,3) =   -0.88781;
  A(4,4) =    1.00294;
  A(4,5) =   -0.63295;
  A(4,6) =    0.53637;
  A(4,7) =   -1.61455;
  A(4,8) =    0.86255;
  A(4,9) =    0.88493;
  A(4,10) =   -0.45795;
  A(4,11) =   -0.96023;
  A(4,12) =    1.78976;
  A(4,13) =    0.31680;
  A(4,14) =   -1.35690;
  A(5,1) =   -2.24210;
  A(5,2) =   -0.41584;
  A(5,3) =    1.03574;
  A(5,4) =   -1.44790;
  A(5,5) =   -0.10965;
  A(5,6) =    1.35520;
  A(5,7) =    0.97862;
  A(5,8) =    1.68439;
  A(5,9) =    0.50804;
  A(5,10) =   -0.91642;
  A(5,11) =   -0.11157;
  A(5,12) =   -1.12676;
  A(5,13) =    0.18238;
  A(5,14) =   -0.44533;
  A(6,1) =   -0.15521;
  A(6,2) =    0.30393;
  A(6,3) =   -0.46746;
  A(6,4) =    1.71455;
  A(6,5) =   -0.52313;
  A(6,6) =    0.13083;
  A(6,7) =   -1.08594;
  A(6,8) =    0.14469;
  A(6,9) =   -0.76766;
  A(6,10) =    1.59178;
  A(6,11) =   -1.37412;
  A(6,12) =    0.73874;
  A(6,13) =    1.26037;
  A(6,14) =    0.17738;
  A(7,1) =   -0.08462;
  A(7,2) =   -0.47582;
  A(7,3) =   -0.42240;
  A(7,4) =    0.32874;
  A(7,5) =    0.45774;
  A(7,6) =    2.13225;
  A(7,7) =   -0.96088;
  A(7,8) =    0.35273;
  A(7,9) =   -0.13499;
  A(7,10) =    0.13492;
  A(7,11) =    0.26117;
  A(7,12) =   -0.02605;
  A(7,13) =   -0.87414;
  A(7,14) =   -1.49695;
  A(8,1) =    0.59622;
  A(8,2) =    0.46725;
  A(8,3) =   -1.46504;
  A(8,4) =   -0.00943;
  A(8,5) =    0.52676;
  A(8,6) =   -0.45699;
  A(8,7) =    0.99116;
  A(8,8) =   -0.30633;
  A(8,9) =    0.58847;
  A(8,10) =    0.07910;
  A(8,11) =    0.36628;
  A(8,12) =    0.84428;
  A(8,13) =   -0.92046;
  A(8,14) =   -1.26145;
  A(9,1) =   -0.54029;
  A(9,2) =    1.05578;
  A(9,3) =    0.37625;
  A(9,4) =   -0.14512;
  A(9,5) =   -0.21476;
  A(9,6) =    0.49829;
  A(9,7) =    0.66953;
  A(9,8) =    0.21016;
  A(9,9) =   -0.36310;
  A(9,10) =    0.55372;
  A(9,11) =    2.77377;
  A(9,12) =    1.20318;
  A(9,13) =   -0.14814;
  A(9,14) =    0.67080;
  A(10,1) =   -0.19409;
  A(10,2) =   -1.72874;
  A(10,3) =    1.21497;
  A(10,4) =    0.81651;
  A(10,5) =   -0.86594;
  A(10,6) =   -2.33380;
  A(10,7) =    1.00021;
  A(10,8) =    0.32066;
  A(10,9) =   -0.17446;
  A(10,10) =    0.22527;
  A(10,11) =   -0.85191;
  A(10,12) =    0.16679;
  A(10,13) =   -1.60160;
  A(10,14) =    0.66610;
  A(11,1) =   -1.52363;
  A(11,2) =    1.46365;
  A(11,3) =   -1.01300;
  A(11,4) =   -1.61628;
  A(11,5) =   -1.61660;
  A(11,6) =    0.30282;
  A(11,7) =   -0.62950;
  A(11,8) =   -0.88456;
  A(11,9) =    0.37883;
  A(11,10) =    2.16172;
  A(11,11) =    0.56202;
  A(11,12) =    1.13580;
  A(11,13) =    0.67701;
  A(11,14) =   -0.44654;
  A(12,1) =    1.39894;
  A(12,2) =    0.27980;
  A(12,3) =    0.15530;
  A(12,4) =   -0.50636;
  A(12,5) =   -0.09203;
  A(12,6) =   -1.72419;
  A(12,7) =   -1.04748;
  A(12,8) =   -0.68468;
  A(12,9) =    0.56841;
  A(12,10) =    1.02306;
  A(12,11) =   -0.53480;
  A(12,12) =   -0.06603;
  A(12,13) =    0.90980;
  A(12,14) =   -1.56507;
  A(13,1) =   -0.66633;
  A(13,2) =    0.61318;
  A(13,3) =   -0.37645;
  A(13,4) =   -1.13804;
  A(13,5) =   -0.89097;
  A(13,6) =   -0.38712;
  A(13,7) =   -1.03391;
  A(13,8) =   -0.64308;
  A(13,9) =    0.36513;
  A(13,10) =    0.54832;
  A(13,11) =   -0.03523;
  A(13,12) =   -0.78828;
  A(13,13) =   -0.67475;
  A(13,14) =   -1.37752;
  A(14,1) =   -1.84847;
  A(14,2) =    0.34907;
  A(14,3) =    0.38296;
  A(14,4) =    0.12063;
  A(14,5) =   -1.24955;
  A(14,6) =    1.98065;
  A(14,7) =   -0.11462;
  A(14,8) =   -0.61500;
  A(14,9) =   -0.19327;
  A(14,10) =    0.47808;
  A(14,11) =   -0.12873;
  A(14,12) =    1.01649;
  A(14,13) =   -0.29699;
  A(14,14) =    0.78186;

  //for (INTEGER i=1; i<=n; ++i) A(i,i) += 3.0*i;

  cout << starbox("/Input matrix A//");
  cout << A;

  max_val = eigen(A,E,ier);
  
  cout << starbox("/Results from call to first eigen function//");
  cout << '\n';
  cout << "\tMaximum eigen value = " << fmt('f',20,8,max_val) << '\n';
  cout << '\n';
  cout << "\tError code = " << ier << '\n';
  cout << '\n';
  cout << "\tEigen values = " << '\n';
  for (INTEGER i=1; i<=n; ++i) {
    cout << '\t' << COMPLEX( E[2*(i-1)+1], E[2*(i-1)+2] ) << '\n';
  }

  max_val = eigen(A,E,X,ier);
  
  cout << starbox("/Results from call to second eigen function//");
  cout << '\n';
  cout << "\tMaximum eigen value = " << fmt('f',20,8,max_val) << '\n';
  cout << '\n';
  cout << "\tError code = " << ier << '\n';
  cout << '\n';
  cout << "\tEigen values = " << '\n';
  for (INTEGER i=1; i<=n; ++i) {
    cout << '\t' << COMPLEX( E[2*(i-1)+1], E[2*(i-1)+2] ) << '\n';
  }
  cout << '\n';
  cout << "\tEigen vectors = " << '\n';
  for (INTEGER j=1; j<=n; ++j) {
    cout << "\tCol " << j << '\n';
    for (INTEGER i=1; i<=n; ++i) {
      cout << '\t' << COMPLEX( X(2*(i-1)+1,j), X(2*(i-1)+2,j) ) << '\n';
    }
  }
  cout << '\n';
  cout << "\tEigen vectors - A * Eigen vectors = " << '\n';
  for (INTEGER j=1; j<=n; ++j) {
    cout << "\tCol " << j << '\n';
    for (INTEGER i=1; i<=n; ++i) {
      COMPLEX sum(0.0,0.0);
      for (INTEGER k=1; k<=n; ++k) {
        sum += COMPLEX(A(i,k),0.0)*COMPLEX(X(2*(k-1)+1,j),X(2*(k-1)+2,j));
      }
      COMPLEX X0 = COMPLEX(X(2*(i-1)+1,j), X(2*(i-1)+2,j));
      COMPLEX X1 = sum/COMPLEX(E[2*(j-1)+1], E[2*(j-1)+2]);
      //cout << '\t' << X0 << "   " << X1 << "   " << X0 - X1 << '\n';
      cout << '\t' << X0 - X1 << '\n';
    }
  }

  return 0;
  }
  
