#include "libscl.h"

using namespace scl;
using namespace std;

int main(int argc, char** argp, char** envp)
{
  INTEGER n = 9;
  INTEGER p = 3;

  realmat X(n,p);
  realmat TX(p,n);

  X(1,1) =  1.0;
  X(2,1) =  1.0;
  X(3,1) =  1.0;
  X(4,1) =  1.0;
  X(5,1) = -8.0;
  X(6,1) =  1.0;
  X(7,1) =  1.0;
  X(8,1) =  1.0;
  X(9,1) =  1.0;

  X(1,2) =  1.0;
  X(2,2) =  1.0;
  X(3,2) =  1.0;
  X(4,2) =  1.0;
  X(5,2) =  0.0;
  X(6,2) = -1.0;
  X(7,2) = -1.0;
  X(8,2) = -1.0;
  X(9,2) = -1.0;

  X(1,3) =  1.0;
  X(2,3) =  1.0;
  X(3,3) = -1.0;
  X(4,3) = -1.0;
  X(5,3) =  0.0;
  X(6,3) =  1.0;
  X(7,3) =  1.0;
  X(8,3) = -1.0;
  X(9,3) = -1.0;

  realmat mu, V;

  INTEGER rank = varcov(X,mu,V,1.e-13);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  cout << '\n';

  n = 4;
  p = 4;

  X.resize(n,p);

  X(1,1) =  1.0;
  X(2,1) =  1.0;
  X(3,1) =  1.0;
  X(4,1) =  1.0;
  
  X(1,2) =  1.0;
  X(2,2) = -1.0;
  X(3,2) =  0.0;
  X(4,2) =  0.0;

  X(1,3) =  2.0;
  X(2,3) = -1.0;
  X(3,3) = -1.0;
  X(4,3) =  0.0;

  X(1,4) =  3.0;
  X(2,4) = -1.0;
  X(3,4) = -1.0;
  X(4,4) = -1.0;

  rank = varcov(X,mu,V,1.e-13);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  cout << '\n';

  INT_32BIT seed = 9834;

  n = 10000000;
  p = 4;

  X.resize(n,p);
  for (INTEGER i=1; i<=n*p; ++i) X[i] = unsk(seed);

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  n = 100000;
  p = 4;

  X.resize(n,p);
  for (INTEGER i=1; i<=n; ++i) {
    X(i,1) = unsk(seed);
    X(i,2) = X(i,1) + unsk(seed);
    X(i,3) = X(i,1) + X(i,2) + unsk(seed);
    X(i,4) = X(i,1) + X(i,2) + X(i,3) + unsk(seed);
  }

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  for (INTEGER i=1; i<=n; ++i) X(i,3) = 1.0;

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  X.resize(n,p);
  for (INTEGER i=1; i<=n*p; ++i) X[i] = 200*ran(seed);

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  TX.resize(p,n);
  for (INTEGER i=1; i<=n; ++i) {
    for (INTEGER j=1; j<=p; ++j) {
      TX(j,i) = X(i,j);
    }
  }

  rank = varcov(T(TX),mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  TX.resize(p,n,0.0);

  rank = varcov(T(TX),mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  n = 0;
  p = 0;

  realmat null;
  X = null;

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  n = 1;
  p = 5;

  X.resize(n,p,5.0);

  rank = varcov(X,mu,V);

  cout << "\t\t n = " << n << '\n';
  cout << "\t\t p = " << p << '\n';
  cout << "\t\t rank = " << rank << '\n';
  cout << '\n';
  cout << "\t\t mu = " << mu << '\n';
  cout << "\t\t V = " << V << '\n';

  return 0;
}
